#!/usr/local/bin/perl
# save_user.cgi
# Create, update or delete a user

require './virtual-server-lib.pl';
&ReadParse();
if ($in{'dom'}) {
	$d = &get_domain($in{'dom'});
	&can_edit_domain($d) || &error($text{'users_ecannot'});
	}
else {
	$access{'local'} || &error($text{'users_ecannot2'});
	}
&lock_user_db();
@users = &list_domain_users($d);
if (!$in{'new'}) {
	# Lookup user details
	($user) = grep { $_->{'user'} eq $in{'old'} &&
			 $_->{'unix'} == $in{'unix'} } @users;
	$user || &error("User does not exist!");
	%old = %$user;
	$mailbox = $d && $d->{'user'} eq $user->{'user'} && $user->{'unix'};
	$user->{'olduser'} = $user->{'user'};
	}
else {
	# Construct initial user object
	$user = &create_initial_user($d);
	}
&error_setup($text{'user_err'});
&require_useradmin();

&build_taken(\%taken, \%utaken);

if ($in{'delete'}) {
	if ($in{'confirm'}) {
		# Get rid of his mail file
		$mailbox && &error($text{'user_edelete'});
		&delete_mail_file($user);

		# Delete the user, his virtusers and aliases
		&delete_user($user, $d);

		if ($user->{'unix'}) {
			# Delete home dir
			&system_logged("rm -rf '$user->{'home'}'")
			  if (-d $user->{'home'} && $user->{'home'} ne "/");
			}

		# Delete in plugins
		foreach $f (@mail_plugins) {
			&plugin_call($f, "mailbox_delete", $user, $d);
			}

		# Delete in other modules
		if ($config{'other_users'}) {
			&foreign_call($usermodule, "other_modules",
				      "useradmin_delete_user", $user);
			}

		$user->{'dom'} = $d->{'dom'};
		&run_post_actions();
		&webmin_log("delete", "user",
			    &remove_userdom($user->{'user'}, $d), $user);
		}
	else {
		# Confirm deletion first
		$ind = $d ? &domain_in($d) : undef;
		&ui_print_header($ind, $text{'user_delete'}, "");

		print &check_clicks_function();
		print "<form action=save_user.cgi>\n";
		print "<input type=hidden name=dom value='$in{'dom'}'>\n";
		print "<input type=hidden name=old value='$in{'old'}'>\n";
		print "<input type=hidden name=unix value='$in{'unix'}'>\n";
		print "<input type=hidden name=delete value=1>\n";
		local ($mailsz) = &mail_file_size($user);
		local ($homesz) = &disk_usage_kb($user->{'home'});
		print "<p>",&text('user_rusure', "<tt>$in{'old'}</tt>",
			  	  &nice_size($mailsz),
				  &nice_size($homesz*1024),
				  "<tt>$user->{'home'}</tt>"),"<p>\n";
		print "<center><input type=submit name=confirm ",
		      "value='$text{'user_deleteok'}' ",
		      "onClick='check_clicks(form)'></center>\n";
		print "</form>\n";

		if ($d) {
			&ui_print_footer("list_users.cgi?dom=$in{'dom'}",
				$text{'users_return'});
			}
		else {
			&ui_print_footer("", $text{'index_return'});
			}
		exit;
		}
	}
else {
	# Verify inputs
	local ($mcount, $mmax) = &count_mailboxes($d);
	if ($in{'new'} && $d && $mmax && $mcount >= $mmax) {
		&error($text{'user_emailboxlimit'});
		}
	if (!$mailbox) {
		$in{'mailuser'} = lc($in{'mailuser'});
		$in{'mailuser'} =~ /^[^ \t:]+$/ || &error($text{'user_euser'});
		if ($user->{'person'}) {
			$in{'real'} =~ /^[^:]*$/ || &error($text{'user_ereal'});
			$user->{'real'} = $in{'real'};
			}
		if ($in{'mailpass_def'}) {
			$user->{'passmode'} = 4;
			}
		else {
			if ($user->{'qmail'}) {
				# Force crypt mode
				local $salt = substr(time(), -2);
				$user->{'pass'} = crypt($in{'mailpass'}, $salt);
				}
			else {
				$user->{'pass'} = &foreign_call(
				  $usermodule, "encrypt_password", $in{'mailpass'});
				}
			$user->{'plainpass'} = $in{'mailpass'};
			$user->{'passmode'} = 3;
			&set_pass_change($user);
			}
		if ($user->{'mailquota'}) {
			# Check qmail quota
			if (!$in{'qquota_def'}) {
				$in{'qquota'} =~ /^\d+$/ ||
					&error($text{'user_eqquota'});
				$user->{'qquota'} = $in{'qquota'};
				}
			else {
				$user->{'qquota'} = 0;
				}
			}

		if ($user->{'mailquotafiles'}) {
			# Check qmail files quota
			if (!$in{'qfiles_def'}) {
				$in{'qfiles'} =~ /^\d+$/ ||
					&error($text{'user_eqfiles'});
				$user->{'qfiles'} = $in{'qfiles'};
				}
			else {
				$user->{'qfiles'} = 0;
				}
			}

		if ($user->{'unix'}) {
			# Check quota inputs
			$qsame = $config{'mail_quotas'} eq $config{'home_quotas'};
			$qedit = &master_admin() || $config{'edit_quota'};
			if ($config{'home_quotas'} && $qedit) {
				if ( $in{'quota'} eq -1 ) {
					$in{'quota'} = $in{'otherquota'};
					}
				$in{'quota_def'} || $in{'quota'} =~ /^\d+$/ ||
					&error($text{'user_equota'});
				}
			if ($config{'mail_quotas'} && !$qsame && $qedit) {
				if ( $in{'mquota'} eq -1 ) {
					$in{'mquota'} = $in{'othermquota'};
					}
				$in{'mquota_def'} || $in{'mquota'} =~ /^\d+$/ ||
					&error($text{'user_equota'});
				}
			}
		}

	# Save extra email addresses
	$eu = $mailbox ? $d->{'user'} : $in{'mailuser'};
	@extra = split(/\s+/, $in{'extra'});
	foreach $e (@extra) {
		$e = lc($e);
		if ($d && $e =~ /^([^\@ \t]+$)$/) {
			$e = "$e\@$d->{'dom'}";
			}
		if ($e !~ /^(\S*)\@(\S+)$/) {
			&error(&text('user_eextra1', $e));
			}
                if ($e eq $eu."\@".$d->{'dom'}) {
                        &error(&text('user_eextra5', $e));
                        }
		local ($eu, $ed) = ($1, $2);
		local $edom = &get_domain_by("dom", $ed);
		$edom && $edom->{'mail'} || &error(&text('user_eextra2', $ed));
		&can_edit_domain($edom) || &error(&text('user_eextra3', $ed));
		}

	if ($d && !$user->{'noprimary'}) {
		$user->{'email'} = $in{'mailbox'} ? $eu."\@".$d->{'dom'}
						  : undef;
		}
	$user->{'extraemail'} = \@extra;
	if ($in{'new'} && &will_send_user_email($d) && !$in{'newmail_def'}) {
		$in{'newmail'} =~ /^\S+$/ || &error($text{'user_enewmail'});
		$newmailto = $in{'newmail'};
		}

	# Validate plugins
	foreach $f (@mail_plugins) {
		$err = &plugin_call($f, "mailbox_validate", $user, \%old, \%in, $in{'new'}, $d);
		&error($err) if ($err);
		}

	if (!$mailbox && !$user->{'fixedhome'} && !$user->{'brokenhome'}) {
		# Find home
		if ((&master_admin() || $config{'edit_homes'}) &&
		    $d && $d->{'home'} && !$in{'home_def'}) {
			$in{'home'} =~ /^\S+$/ && $in{'home'} !~ /\.\./ ||
				&error($text{'user_ehome'});
			# Custom home directory for mailbox user
			$home = "$d->{'home'}/$in{'home'}";
			}
		elsif ($d) {
			# Auto home directory for mailbox user
			$home = "$d->{'home'}/$config{'homes_dir'}/$in{'mailuser'}";
			}
		else {
			# Auto home directory for local user
			$home = &useradmin::auto_home_dir(
				$home_base, $in{'mailuser'}, $config{'localgroup'});
			}
		}

        # Update secondary groups
        %cangroups = map { $_, 1 } (&allowed_secondary_groups($d),
                                    @{$user->{'secs'}});
        @secs = split(/\0/, $in{'groups'});
        foreach my $g (@secs) {
                $cangroups{$g} || &error(&text('user_egroup', $g));
                }
        $user->{'secs'} = [ @secs ];

	# Create or update the user
	if ($in{'new'}) {
		# Set new user parameters
		if ($user->{'unix'}) {
			# UID needs to be unique
			$user->{'uid'} = &allocate_uid(\%taken);
			}
		else {
			# UID is same as domain for Qmail users
			$user->{'uid'} = $d->{'uid'};
			}
		$user->{'gid'} = $d ? $d->{'gid'} :
				      getgrnam($config{'localgroup'});

		# Check for clash within this domain
		($clash) = grep { $_->{'user'} eq $in{'mailuser'} &&
			  	  $_->{'unix'} == $user->{'unix'} } @users;
		$clash && &error($text{'user_eclash2'});

		if ($user->{'unix'}) {
			if (&master_admin() || $config{'edit_ftp'}) {
				# Shell can be set based on FTP flag
				$user->{'shell'} =
					$in{'ftp'} == 1 ? $config{'ftp_shell'} :
					$in{'ftp'} == 2 ? $config{'jail_shell'} :
							  $config{'shell'};
				}
			elsif ($in{'new'}) {
				# For new users, shell is always non-FTP
				$user->{'shell'} = $config{'shell'};
				}
			}
		if (!$user->{'fixedmode'}) {
			$user->{'home'} = $home;
			}

		if (($utaken{$in{'mailuser'}} || ($d && $config{'append'})) &&
		    !$user->{'noappend'}) {
			# Need to append domain name
			if ($d) {
				# Add group name
				$user->{'user'} = &userdom_name($in{'mailuser'},$d);
				}
			else {
				# No domain to add, so give up!
				&error($text{'user_eclash2'});
				}
			}
		else {
			$user->{'user'} = $in{'mailuser'};
			}
		$user->{'passmode'} = 3;
		$user->{'plainpass'} = $in{'mailpass'};

		if ($d && $user->{'unix'}) {
			# Check for a Unix clash
			if ($utaken{$user->{'user'}} ||
			    &check_clash($in{'mailuser'}, $d->{'dom'})) {
				&error($text{'user_eclash'});
				}
			}

		# Check if any extras clash
		foreach $e (@extra) {
			$e =~ /^(\S*)\@(\S+)$/;
			if (&check_clash($1, $2)) {
				&error(&text('user_eextra4', $e));
				}
			}

		# Check if the name is too long
		if ($user->{'unix'} &&
		    ($lerr = &too_long($user->{'user'}))) {
			&error($lerr);
			}

		# Check if home directory already exists
		if (-e $home) {
			&error(&text('user_emkhome', $home));
			}

		# Set mail file location
		if ($user->{'qmail'}) {
			&userdom_substitutions($user, $d);
			$user->{'mailstore'} =
			 &substitute_template($config{'ldap_mailstore'}, $user);
			}

		if (!$user->{'noalias'}) {
                        # Save alias
                        @values = &parse_alias(undef, $user->{'user'},
                                               undef, "user", $d);
                        $user->{'to'} = @values ? \@values : undef;
                        }

		# Create the user and virtusers and alias
		&create_user($user, $d);

		if ($home) {
			# Create his homedir
			&system_logged("mkdir -p '$home'");
			&system_logged("chown $user->{'uid'}:$user->{'gid'} '$home'");
			&system_logged("chmod 755 '$home'");

			# Copy files into homedir
			&copy_skel_files($config{'mail_skel'}, $user, $home);
			}

		# Create an empty mail file
		&create_mail_file($user);

		# Send an email upon creation
		@erv = &send_user_email($d, $user, $newmailto);
		}
	else {
		# Check if any extras clash
		%oldextra = map { $_, 1 } @{$old{'extraemail'}};
		foreach $e (@extra) {
			$e =~ /^(\S*)\@(\S+)$/;
			if (!$oldextra{$e} && &check_clash($1, $2)) {
				&error(&text('user_eextra4', $e));
				}
			}

		# For any user except the domain owner, update his home and shel
		if (!$mailbox) {
			# Check if new homedir exists
			if (-e $home && $user->{'home'} ne $home) {
				&error(&text('user_emkhome', $home));
				}

			# Update user parameters (handle rename and .group)
			if ($in{'mailuser'} ne $in{'oldpop3'}) {
				# Check for a clash in this domain
				($clash) = grep { $_->{'user'} eq $in{'mailuser'} &&
				  $_->{'unix'} == $user->{'unix'} } @users;
				$clash && &error($text{'user_eclash2'});

				# Has been renamed .. check for a username clash
				if ($d && ($utaken{$in{'mailuser'}} ||
					   $config{'append'}) &&
				    !$user->{'noappend'}) {
					# New name has to include group
					$user->{'user'} =
						&userdom_name($in{'mailuser'}, $d);
					}
				else {
					# Can rename without the dot
					$user->{'user'} = $in{'mailuser'};
					}

				# Check if the name is too long
				if ($lerr = &too_long($user->{'user'})) {
					&error($lerr);
					}

				# Check for a virtuser clash too
				if ($d && &check_clash($in{'mailuser'},
						       $d->{'dom'})) {
					&error($text{'user_eclash'});
					}
				}

			# Rename homedir
			if ($user->{'home'} ne $home && -d $user->{'home'} &&
			    !$user->{'fixedhome'} && !$user->{'brokenhome'}) {
				&system_logged("mv '$user->{'home'}' '$home'");
				$user->{'home'} = $home;
				}

			# Update shell
			if (defined($in{'ftp'})) {
				$user->{'shell'} =
				    $in{'ftp'} == 1 ? $config{'ftp_shell'} :
				    $in{'ftp'} == 2 ? $config{'jail_shell'} :
						      $config{'shell'};
				}

			# Set mail file location
			if ($user->{'qmail'}) {
				local $store = &substitute_template(
					$config{'ldap_mailstore'}, $user);
				$user->{'mailstore'} = $store;
				}

			# Rename his mail file (if needed)
			&rename_mail_file($user, \%old);
			}

		if (!$user->{'noalias'}) {
                        # Save aliases
                        @values = &parse_alias(undef, $user->{'user'},
                                        $old{'to'}, "user", $d);
                        $user->{'to'} = @values ? \@values : undef;
                        }

		# Update the user and any virtusers and aliases
		&modify_user($user, \%old, $d);
		}

	if (!$mailbox && ($qedit || $in{'new'}) && $user->{'unix'}) {
		# Actually update quotas, if allowed
		@defmquota = split (/ /,$config{'defmquota'});
		if ($config{'home_quotas'}) {
			# Set his home directory quota
			&set_quota($user->{'user'}, $config{'home_quotas'},
			   !$qedit ? $defmquota[0] :
			    $in{'quota_def'} ? 0 :
				&quota_parse('quota', $config{'home_quotas'}));
			}
		if ($config{'mail_quotas'} && !$qsame) {
			# Set his home directory quota
			&set_quota($user->{'user'}, $config{'mail_quotas'},
			   !$qedit ? $defmquota[0] :
			   $in{'mquota_def'} ? 0 :
				&quota_parse('mquota', $config{'mail_quotas'}));
			}
		}

	# Run plugin save functions
	foreach $f (@mail_plugins) {
		&plugin_call($f, "mailbox_save", $user, \%old, \%in, $in{'new'}, $d);
		}

	# Call other module functions
	if ($config{'other_users'}) {
		if ($in{'new'}) {
			&foreign_call($usermodule, "other_modules",
				      "useradmin_create_user", $user);
			}
		else {
			&foreign_call($usermodule, "other_modules",
				      "useradmin_modify_user", $user, \%old);
			}
		}

	&run_post_actions();
	$user->{'dom'} = $d->{'dom'};
	&webmin_log($in{'new'} ? "create" : "modify", "user",
		    &remove_userdom($user->{'user'}, $d), $user);
	}
&unlock_user_db();
&redirect($d ? "list_users.cgi?dom=$in{'dom'}" : "index.cgi");

